module Operations
  class EnforcedController < ActionController::Base

    include ApplicationHelper
    include OperationsHelper

    before_action :check_operations

    rescue_from Operations::Errors::NotLoggedInError, with: :go_to_login
    rescue_from Operations::Errors::NotAuthorizedError, with: :render_403_error

    def render_403_error(err)
      Rails.logger.info "### Operations::Error : #{err} (#{err.class}) ###"
      respond_to do |format|
        format.html {redirect_back(fallback_location: root_path, alert: "#{err}")}
        format.xml {render xml: {message: "#{err}"}, status: 403}
        format.json {render json: {message: "#{err}"}, status: 403}
      end
    end

    def go_to_login(err)
      respond_to do |format|
        format.html {
          if Operations::Config.sign_in_path
            redirect_to(Operations::Config.sign_in_path, alert: 'Please login first.')
          else
            redirect_back(fallback_location: root_path, alert: "#{err}")
          end
        }
        format.xml {render xml: {message: "#{err}"}, status: 403}
        format.json {render json: {message: "#{err}"}, status: 403}
      end
    end

    private
    def check_operations
      controller = params[:controller]; action = params[:action]
      if respond_to?(:current_user)
        warn "Enforcing #{controller}:#{action}..."
        Operations::Enforcer.enforce(controller, action, current_user)
      end
    end

  end
end
