module Operations
  module Config
    # Returns a list of user roles straight from the config file
    mattr_accessor :user_roles
    @@user_roles = [
      {name: :admin, description: 'Administrator', value: 0, db_value: 0},
      {name: :guest, description: 'Anyone', value: 3, db_value: 1},
      {name: :regular, description: 'Regular user', value: 2, db_value: 2},
      {name: :technician, description: 'Technician', value: 1, db_value: 3},
      {name: :all, description: 'Anyone on the system', value: -1},
      {name: :nobody, description: 'No one on the system', value: -2}
    ]

    mattr_accessor :operation_name_regex
    @@operation_name_regex = %r{\w}

    mattr_accessor :operation_uuid_algorithm
    @@operation_uuid_algorithm = OpenSSL::Digest::SHA256

    # Returns a list of Operation instances for convenience
    mattr_accessor :operations_list
    @@operations_list = []

    # Specifies how the operations should be enforced within the app
    mattr_accessor :enforcements
    @@enforcements = []

    # Defines the default path to go to should the user not be logged in
    mattr_accessor :sign_in_path
    @@sign_in_path = nil

    class << self
      def operation_scope_regex
        %r{\A(#{user_roles.map{|o| o[:name]}.join('|')})\z}
      end

      def operation_scope_regex=(*args)
        raise 'Operations::Config does not allow to set the variable operation_name_regex'
      end

      def get_sign_in_path
        if sign_in_path.nil?
          warn "Warning! You have not set the Operations::Config.sign_in_path variable!"
          return nil
        end
        Rails.application.routes.recognize_path sign_in_path
      rescue ActionController::RoutingError
        {controller: nil, action: nil}
      end
    end

    # A simple way to setup Operations.
    def self.setup
      yield self
    end

    def self.get_operations_list
      operations_list
          .map{|operation|
            Operations::Operation.new(**operation)}
          .select{|operation| operation.is_valid?}
    end
  end
end
